package tut.tutorial.tutorial;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Build;
import android.service.notification.NotificationListenerService;
import android.service.notification.StatusBarNotification;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;

import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Set;
import java.util.StringTokenizer;

import de.robv.android.xposed.IXposedHookLoadPackage;
import de.robv.android.xposed.XC_MethodHook;
import de.robv.android.xposed.XposedBridge;
import de.robv.android.xposed.XposedHelpers;
import de.robv.android.xposed.callbacks.XC_LoadPackage;

/**
 * Created by uwek on 24.02.2016.
 */
public class Module  implements IXposedHookLoadPackage
{
    public static final String KIWI = Module.class.getPackage().getName();
    public static final String STATUS = "<status>";
    //package and classes
    public static final String SYSTEMUI = "com.android.systemui";
    public static final String PHONE_STATUS_BAR = SYSTEMUI +".statusbar.phone.PhoneStatusBar";
    public static final String BASE_STATUS_BAR = SYSTEMUI +".statusbar.BaseStatusBar";
    //hooked methods
    public static final String MAKE_STATUS_BAR_VIEW = "makeStatusBarView";
    public static final String ADD_NOTIFICATION = "addNotification";
    public static final String REMOVE_NOTIFICATION = "removeNotification";
    public static final String UPDATE_NOTIFICATION = "updateNotification";
    public static final String UPDATE_NOTIFICATION_ICONS = "updateNotificationIcons";

    //accessed methods and fields
    public static final String CREATE_NOTIFICATION_VIEWS = "createNotificationViews";
    public static final String M_CONTEXT = "mContext";
    public static final String M_NOTIFICATION_ICONS = "mNotificationIcons";
    public static final String M_SYSTEM_ICON_AREA = "mSystemIconArea";
    public static final String ICON = "icon";

    private Set<String> i_pack = new HashSet<>();
    private LinkedHashMap<String, View> i_icons = new LinkedHashMap<>();
    private LinearLayout i_status;
    private int i_sdk;

    @Override
    public void handleLoadPackage(XC_LoadPackage.LoadPackageParam lpparam) throws Throwable
    {
        if (SYSTEMUI.equals(lpparam.packageName))
        {
            i_pack = new HashSet<>();
            i_sdk = Build.VERSION.SDK_INT;
            XposedBridge.log(KIWI + " started on Sdk " + i_sdk);

            XposedHelpers.findAndHookMethod(PHONE_STATUS_BAR, lpparam.classLoader, MAKE_STATUS_BAR_VIEW, new XC_MethodHook()
            {
                @Override
                protected void afterHookedMethod(MethodHookParam param) throws Throwable
                {
                    final Context ctx = (Context) XposedHelpers.getObjectField(param.thisObject, M_CONTEXT);
                    final Object psb = param.thisObject;
                    IntentFilter filter = new IntentFilter();
                    filter.addAction(SettingsActivity.ACTION_PACK_ADD);
                    filter.addAction(SettingsActivity.ACTION_PACK_REMOVE);
                    filter.addAction(SettingsActivity.ACTION_ALL_PACK);
                    ctx.registerReceiver(new BroadcastReceiver()
                    {
                        @Override
                        public void onReceive(Context context, Intent intent)
                        {
                            if (intent.getAction().equals(SettingsActivity.ACTION_PACK_ADD))
                            {
                                String s = intent.getStringExtra(SettingsActivity.PACKAGE);
                                i_pack.add(s);
                                XposedBridge.log(KIWI + " Broadcast add got " + intent.getStringExtra(SettingsActivity.PACKAGE));
                                if (STATUS.equals(s))
                                {
                                    XposedHelpers.callMethod(psb, UPDATE_NOTIFICATION_ICONS, new Class[0]);
                                }
                            }
                            else if (intent.getAction().equals(SettingsActivity.ACTION_PACK_REMOVE))
                            {
                                String s = intent.getStringExtra(SettingsActivity.PACKAGE);
                                i_pack.remove(s);
                                XposedBridge.log(KIWI + " Broadcast remove got " + intent.getStringExtra(SettingsActivity.PACKAGE));
                                if (STATUS.equals(s))
                                {
                                    XposedHelpers.callMethod(psb, UPDATE_NOTIFICATION_ICONS, new Class[0]);
                                }
                            }
                            else if (intent.getAction().equals(SettingsActivity.ACTION_ALL_PACK))
                            {
                                HashSet<String> pack = new HashSet<String>();
                                StringTokenizer tok = new StringTokenizer(intent.getStringExtra(SettingsActivity.PACKAGES), " ");
                                while (tok.hasMoreTokens())
                                    pack.add(tok.nextToken());
                                XposedBridge.log(KIWI + " Broadcast all got " + intent.getStringExtra(SettingsActivity.PACKAGES));
                                i_pack = pack;
                            }
                        }
                    }, filter);
                    try
                    {
                        Intent i = new Intent(SettingsActivity.ACTION_SEND_PACKS);
                        i.setPackage(KIWI);
                        ctx.startService(i);
                        XposedBridge.log(KIWI + " " + MAKE_STATUS_BAR_VIEW + " sent intent...");
                    }
                    catch (Throwable t)
                    {
                        XposedBridge.log(KIWI + " " + MAKE_STATUS_BAR_VIEW + " failed intent: " + t);
                    }
                    ViewGroup iconArea = (ViewGroup) XposedHelpers.getObjectField(psb, M_SYSTEM_ICON_AREA);
                    i_status = new LinearLayout(ctx);
                    iconArea.addView(i_status, 0);
                }
            });

            XposedHelpers.findAndHookMethod(PHONE_STATUS_BAR, lpparam.classLoader, ADD_NOTIFICATION,
                    StatusBarNotification.class, NotificationListenerService.RankingMap.class, new XC_MethodHook()
                    {
                        @Override
                        protected void beforeHookedMethod(MethodHookParam param) throws Throwable
                        {
                            StatusBarNotification sbn = (StatusBarNotification) param.args[0];
                            XposedBridge.log(KIWI + " " + ADD_NOTIFICATION + " got " + sbn.getPackageName() + ":" + sbn.getKey() + ":" + sbn.getNotification().icon);
                            if (!sbn.isClearable())
                            {
                                if (i_pack.contains(sbn.getPackageName()))
                                {
                                    param.setResult(null);
                                    Object o = XposedHelpers.callMethod(param.thisObject, CREATE_NOTIFICATION_VIEWS, new Class[]{StatusBarNotification.class}, sbn);
                                    View icon = (View) XposedHelpers.getObjectField(o, ICON);
                                    XposedBridge.log(KIWI + " " + ADD_NOTIFICATION + " got icon " + icon);
                                    i_icons.put(sbn.getKey(), icon);
                                    XposedHelpers.callMethod(param.thisObject, UPDATE_NOTIFICATION_ICONS, new Class[0]);
                                }
                                else
                                {
                                    XposedBridge.log(KIWI + " " + ADD_NOTIFICATION + " did not find " + sbn.getPackageName() + " in " + i_pack);
                                }
                            }
                            else
                            {
                                XposedBridge.log(KIWI + " " + ADD_NOTIFICATION + " found clearable " + sbn.getPackageName());
                            }
                        }
                    });
            XposedHelpers.findAndHookMethod(PHONE_STATUS_BAR, lpparam.classLoader, REMOVE_NOTIFICATION,
                    String.class, NotificationListenerService.RankingMap.class, new XC_MethodHook()
                    {
                        @Override
                        protected void beforeHookedMethod(MethodHookParam param) throws Throwable
                        {
                            String key = (String) param.args[0];
                            XposedBridge.log(KIWI + " " + REMOVE_NOTIFICATION + " got " + key);
                            if (i_icons.containsKey(key))
                            {
                                param.setResult(null);
                                i_icons.remove(key);
                                XposedBridge.log(KIWI + " " + REMOVE_NOTIFICATION + " removed " + key);
                                XposedHelpers.callMethod(param.thisObject, UPDATE_NOTIFICATION_ICONS, new Class[0]);
                            }
                        }
                    });

            XposedHelpers.findAndHookMethod(BASE_STATUS_BAR, lpparam.classLoader, UPDATE_NOTIFICATION, StatusBarNotification.class, NotificationListenerService.RankingMap.class, new XC_MethodHook()
            {
                @Override
                protected void beforeHookedMethod(MethodHookParam param) throws Throwable
                {
                    StatusBarNotification sbn = (StatusBarNotification) param.args[0];
                    XposedBridge.log(KIWI + " " + UPDATE_NOTIFICATION + " got " + sbn.getPackageName() + ":" + sbn.getKey() + ":" + sbn.getNotification().icon);
                    if (i_icons.containsKey(sbn.getKey()))
                    {
                        param.setResult(null);
                        Object o = XposedHelpers.callMethod(param.thisObject, CREATE_NOTIFICATION_VIEWS, new Class[]{StatusBarNotification.class}, sbn);
                        View icon = (View) XposedHelpers.getObjectField(o, "icon");
                        XposedBridge.log(KIWI + " " + UPDATE_NOTIFICATION + " stored icon " + icon);
                        i_icons.put(sbn.getKey(), icon);
                        XposedHelpers.callMethod(param.thisObject, UPDATE_NOTIFICATION_ICONS, new Class[0]);
                    }
                }
            });
            XposedHelpers.findAndHookMethod(PHONE_STATUS_BAR, lpparam.classLoader, UPDATE_NOTIFICATION_ICONS, new XC_MethodHook()
            {
                @Override
                protected void afterHookedMethod(MethodHookParam param) throws Throwable
                {
                    XposedBridge.log(KIWI + " " + UPDATE_NOTIFICATION_ICONS + " called");
                    i_status.removeAllViews();
                    if (i_pack.contains(STATUS))
                    {
                        for (View icon : i_icons.values())
                            i_status.addView(icon, 0);
                    }
                    else
                    {
                        LinearLayout ll = (LinearLayout) XposedHelpers.getObjectField(param.thisObject, M_NOTIFICATION_ICONS);
                        for (View icon : i_icons.values())
                            ll.addView(icon);
                    }
                }
            });
        }
    }
}
